<?php
declare(strict_types=1);

namespace FiloBlu\Refilo\Remote\Entity\Provider;

use FiloBlu\Refilo\Helper\Url;
use FiloBlu\Refilo\Model\Config\Seo\HrefLangConfig;
use FiloBlu\Refilo\Remote\Entity\EntityInterface;
use FiloBlu\Refilo\Remote\Entity\EntityProviderInterface;
use FiloBlu\Refilo\Remote\Entity\UrlRewrite;
use FiloBlu\Refilo\Remote\Entity\UrlRewriteFactory;
use LogicException;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Store\Api\Data\StoreInterface;
use Magento\Store\Api\Data\WebsiteInterface;
use Magento\Store\Model\Store;
use Magento\Store\Model\Website;

/**
 *
 */
abstract class AbstractUrlProvider extends AbstractFromQueryProvider
{
    /**
     * @var HrefLangConfig
     */
    private $hrefLangConfig;

    /**
     * @var UrlRewriteFactory
     */
    private $urlRewriteFactory;

    /**
     * @var Url
     */
    private $urlHelper;

    /**
     * @var Store|StoreInterface| null
     */
    private $store = null;

    /**
     * @param ResourceConnection $resourceConnection
     * @param HrefLangConfig $hrefLangConfig
     * @param ScopeConfigInterface $scopeConfig
     * @param Url $urlHelper
     * @param UrlRewriteFactory $urlRewriteFactory
     */
    public function __construct(ResourceConnection   $resourceConnection,
                                HrefLangConfig       $hrefLangConfig,
                                ScopeConfigInterface $scopeConfig,
                                Url                  $urlHelper,
                                UrlRewriteFactory    $urlRewriteFactory
                               )
    {
        parent::__construct($resourceConnection, $scopeConfig);
        $this->hrefLangConfig = $hrefLangConfig;
        $this->urlRewriteFactory = $urlRewriteFactory;
        $this->urlHelper = $urlHelper;
    }

    /**
     * @return Url
     */
    public function getUrlHelper(): Url
    {
        return $this->urlHelper;
    }

    /**
     * @param StoreInterface $store
     * @return EntityProviderInterface
     */
    public function forStore(StoreInterface $store): EntityProviderInterface
    {
        $this->store = $store;
        return $this;
    }

    /**
     * @param WebsiteInterface| Website $website
     * @return EntityProviderInterface
     */
    public function forWebsite(WebsiteInterface $website): EntityProviderInterface
    {
        throw new LogicException('Website filtering is not supported');
    }

    /**
     * @return StoreInterface|Store|null
     */
    public function getStore()
    {
        return $this->store;
    }

    /**
     * @return bool
     */
    public function hasStore(): bool
    {
        return $this->store !== null;
    }

    /**
     * @param array $row
     * @return EntityInterface
     * @throws NoSuchEntityException
     * @throws LocalizedException
     */
    public function adapt(array $row): EntityInterface
    {
        /** @var UrlRewrite $urlRewrite */
        $urlRewrite = $this->getUrlRewriteFactory()->create(['data' => $row]);
        $urlRewrite->setHreflang($this->urlHelper->getHreflang($urlRewrite));
        $urlRewrite->setCanonical($this->urlHelper->getCanonicalUrl($urlRewrite));
        return $urlRewrite;
    }

    /**
     * @return UrlRewriteFactory
     */
    public function getUrlRewriteFactory(): UrlRewriteFactory
    {
        return $this->urlRewriteFactory;
    }
}
